import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Observer;
import java.net.URL;

public class AlgoritmiAnimati extends JApplet implements Costanti {

	public AlgoritmiAnimati(URL codiceBase) {
		this.codiceBase = codiceBase;
	}
		
	public static void main(String args[]) {
		theApp = new AlgoritmiAnimati(null);
		theApp.init();
	}
	
	public void esci() {
		if (theApp != null)
			System.exit(0);
	}
	
	public void init() {
		try {javax.swing.UIManager.setLookAndFeel(javax.swing.UIManager.getSystemLookAndFeelClassName());} catch(Exception e) {}
		
		model	= new AlgoritmiModel(this,ALGORITMO_DEFAULT);

		finestra = new AlgoritmiFrame("LiveSort",this);
		
		Toolkit kitto 			= finestra.getToolkit();
		Dimension dimensione	= kitto.getScreenSize();
		
		finestra.setBounds(dimensione.width/4, dimensione.height/4,
						   dimensione.width/2, dimensione.height/2);
						   
		view	= new AlgoritmiView(this);
		model.addObserver((Observer)view);
		
		GridBagLayout gridBag = new GridBagLayout();
		JPanel pannelloCentro = new JPanel(gridBag);
		GridBagConstraints limiti = new GridBagConstraints();
		limiti.weightx = limiti.weighty = 10;
		limiti.fill = limiti.BOTH;
		gridBag.setConstraints(view,limiti);
		pannelloCentro.add(view);
		limiti.gridx = 0;
		limiti.gridwidth = limiti.REMAINDER;
		limiti.weighty = 0.1;
		JPanel pannelloMessaggio = new JPanel(new FlowLayout(FlowLayout.LEFT));
		pannelloMessaggio.setBorder(BorderFactory.createTitledBorder(new EtchedBorder(),"Descrizione"));
		AlgoritmiView.AlgoritmiMessaggio messaggio = view.new AlgoritmiMessaggio("Premi >> per iniziare");
		pannelloMessaggio.add(messaggio);
		gridBag.setConstraints(pannelloMessaggio,limiti);
		pannelloCentro.add(pannelloMessaggio);
		
		finestra.getContentPane().add(pannelloCentro,BorderLayout.CENTER);
		pannelloCentro.setPreferredSize(new Dimension(300,350));
		
		finestra.pack();
		finestra.setVisible(true);
				
		ListaStati.setView(view);
		ListaStati.resetta();
	}

	int getNElementi() {
		return nElementi;
	}
	
	Color getColore() {
		return colore;
	}
	
	void setColore(Color colore) {
		this.colore = colore;
	}
	
	void setIntervallo(long intervallo) {
		this.intervallo = intervallo;
		model.setIntervallo(intervallo);
	}
	
	long getIntervallo() {
		if (this.turbo) {
			return this.INTERVALLO_TURBO;
		}
		return this.intervallo;
	}
	
	AlgoritmiFrame getFinestra() {
		return finestra;
	}
	
	AlgoritmiView getView() {
		return view;
	}
	
	AlgoritmiModel getModel() {
		return model;
	}
	
	void setTurbo(boolean turbo) {
		this.turbo = turbo;
		if (turbo) {
			model.setIntervallo(this.INTERVALLO_TURBO);
		} else {
			model.setIntervallo(this.intervallo);
		}
	}
	
	boolean getTurbo() {
		return this.turbo;
	}
	
	void avvisaTerminato() {
		finestra.resettaPulsanti();
	}
	
	public void setVettore(int[] vettore) {
		this.vettore = vettore;
		this.model.setVettore(vettore);  //Vorrei evitarlo fin che il sorting e' in esecuzione, perche' puo' accadere che avessi la finestra dell'editor del vettore aperta fin che il sorting e' in esecuzione e, accettando un vettore, il vettore su cui stia lavorando il sorting cambierebbe improvvisamente! Quindi lascio che il vettore venga preso quando si avvia il sorting (con il metodo AlgoritmiAnimati.getVettore())
										 //Fatto. V. Sorting.setVettore: evita di settare il vettore se thread != null (e quindi e' in esecuzione il sorting)!
	}
	
	public int[] getVettore() {
		return this.vettore;
	}
	
	public URL getCodiceBase() {
		return this.codiceBase;
	}
		
	private static AlgoritmiAnimati theApp 		= null;
	private AlgoritmiFrame finestra 			= null;
	private AlgoritmiModel model 				= null;
	private AlgoritmiView view 					= null;
	private boolean turbo						= TURBO_INIZIALE;
	
	private URL codiceBase						= null;
	
	private int nElementi 						= N_ELEMENTI_DEFAULT;
	private Color colore 						= COLORE_DEFAULT;
	private int[] vettore						= VettoreFrame.vettoreRandom(nElementi,1,nElementi);
	private long intervallo						= INTERVALLO_DEFAULT;
}
