import javax.swing.*;
import java.awt.*;
import java.awt.geom.*;

public class HeapSort extends Sorting {

	public HeapSort(int[] vettore, long intervallo) {
		super(intervallo);
		int nElementi = vettore.length;
		this.vettore = new int[nElementi];
		for (int i=0;i<nElementi;i++) {
			this.vettore[i] = vettore[i];
		}
		
		String[] codice = {
			"void maximize(int[] a, int i, int j) {",
			"   int root = i;",
			"   boolean done = false;",
			"   int value = a[i];",
			"   while (2*root <= j && !done) {",
			"       int posMax = 2 * root;",
			"       if (posMax < j)",
			"           if (a[posMax+1] > a[posMax])",
			"               posMax++;",
			"       if (a[posMax] > value) {",
			"           a[root] = a[posMax];",
			"           root = posMax;",
			"       } else {",
			"           done = true;",
			"       }",
			"   }",
			"   a[root] = value;",
			"}",
			"",
			"void makeHeap(int[] a, int n) {",
			"   for(int i=(n/2);i>=0;i--)",
			"       maximize(a, i, n);",
			"}",
			"",
			"void maxSort(int[] a, int n) {",
			"   int i = n;",
			"   while (i > 0) {",
			"       swap(a,0,i);",
			"       i = i-1;",
			"       maximize(a, 0, i);",
			"   }",
			"}",
			"",
			"void heapSort(int[] a, int high) {",
			"   makeHeap(a, high);",
			"   maxSort(a, high);",
			"}",
		};
		this.codice = codice;
		
		String[] spiegazione = {
			"Entra in maximize con il vettore e gli estremi dell'intervallo da 'massimizzare'",
			"Alla fine, in root ci sara' la nuova posizione dell'elemento che inizialmente era alla posizione i",
			"Quando avremo trovato il posto per l'elemento (cioe' quando non ci saranno piu' conflitti 'padre-figlio'), done varra' true",
			"In value si tiene salvato il valore dell'elemento per cui bisogna trovare il posto appropriato",
			"Continua finche' almeno un 'figlio' di root sara' nell'intervallo o finche' non avrai trovato un posto per value",
			"posMax contiene l'indice del 'figlio' sinistro di root",
			"Se root ha anche un 'figlio' destro,",
			"E il figlio destro e' il maggiore",
			"Allora posMax punta al figlio destro",
			"Se il valore del figlio maggiore e' superiore a value",
			"Allora questo posto non va bene: sposta il figlio nel posto di root",
			"E ricomincia prendendo root = l'indice del figlio maggiore",
			"Se invece value e' >= al valore del figlio maggiore",
			"Allora abbiamo trovato un posto per value",
			"",
			"Continua finche' non si e' trovato un posto o non si e' all'estremo superiore dell'intervallo",
			"Metto value nel posto root, dove non crea conflitti 'padre-figlio'",
			"Fine di maximize",
			"",
			"Entra in makeHeap con il vettore e l'indice dell'estremo superiore dell'intervallo",
			"Comincia dall'indice del primo nodo che abbia figli (il 'padre' dell'ultimo elemento) e prosegui a ritroso finche' non sono stati presi in considerazione tutti i 'padri'",
			"Esegui maximize nell'intervallo {i,n}",
			"Fine di makeHeap",
			"",
			"Entra in maxSort con il vettore e l'indice dell'estremo superiore dell'intervallo",
			"i e' l'indice della posizione dove va messo l'elemento piu' grande trovato nello heap",
			"Esegui il ciclo finche' i non e' arrivato alla prima posizione del vettore",
			"Scambia il primo elemento del vettore (ovvero la radice - il massimo - dello heap) con l'i-esimo elemento",
			"Prosegui al prossimo elemento",
			"Esegui maximize tra 0 e i, per restaurare la heap-property",
			"Continua finche' non hai 'svuotato' lo heap",
			"Fine di maxSort",
			"",
			"Entra in HeapSort con il vettore e l'indice massimo",
			"Crea lo heap",
			"Ordina il vettore, estraendo ogni volta l'elemento maggiore dallo heap",
			"Fine dell' HeapSort",
  		};
  		this.spiegazione = spiegazione;
	}
	
	
	public void draw(Graphics g1, Rectangle area) {
		this.area = new Rectangle(area.x,area.y-10,area.width,area.height/2-10); 
		  //^^ Per l' HeapSort l'area di disegno del vettore e' sopra e per lo heap e' sotto
		Graphics2D g = (Graphics2D)g1;
		disegnaVettore(g1,vettore,this.area,xInizio,yInizio,elementoEvidenziato1,elementoEvidenziato2,animazione);

		g.draw(new Line2D.Double(area.x,area.y+area.height/2-5,area.x + area.width,area.y+area.height/2-5));

		Albero.disegnaAlbero(g1,new Rectangle(area.x,area.y+area.height/2-5,area.width,area.height/2+5));
	}

	private void inserisciHeap(int[] vettore, int low, int high, int indiceEvidenziato) {
		Albero.cancella();
		int i = low;
		int livello = 1;
		Nodo nodoInserito = null;
		while (i <= high) {
			if ((nodoInserito=Albero.aggiungiNodo(Integer.toString(vettore[i]),livello,Albero.radice))==null) {
				nodoInserito = Albero.aggiungiNodo(Integer.toString(vettore[i]),++livello,Albero.radice);
			}
			if (i == indiceEvidenziato) {
				Albero.evidenziaNodo(nodoInserito);
			}
			i++;
		}
	}		

	void maximize(int[] a, int i, int j) {
		evidenziaLinea(0);
		
		elementoEvidenziato1 = i; elementoEvidenziato2 = -1;
		evidenziaLinea(1);
		int root = i;
		
		evidenziaLinea(2);
		boolean done = false;
		
		evidenziaLinea(3);
		if (this.intervallo != INTERVALLO_TURBO) {
			setMessaggio("Devo trovare un posto per " + a[i]);
			pausa(intervallo);
		} else {
			setMessaggio("Instauro la heap-property tra gli elementi " + (i+1) + " e " + (j+1) + "");
			elementoEvidenziato1 = i;
			elementoEvidenziato2 = j;
		}
		int value = a[i];
		
		evidenziaLinea(4);
		while (2*root <= j && !done) {
			
			evidenziaLinea(5);
			int posMax =  2 * root;
			if (this.intervallo != INTERVALLO_TURBO) {
				setMessaggio("Il figlio maggiore di " + a[root] + " per il momento e' " + a[posMax]);
				elementoEvidenziato2 = posMax;
				pausa(intervallo);
			} else {
				setMessaggio("Instauro la heap-property tra gli elementi " + (i+1) + " e " + (j+1) + "");
				elementoEvidenziato1 = i;
				elementoEvidenziato2 = j;
			}
			
			
			evidenziaLinea(6);
			if (this.intervallo != INTERVALLO_TURBO) {
				setMessaggio(Integer.toString(posMax+1) + " < " + (j+1) + " ? ");
				pausa(intervallo);
			} else {
				setMessaggio("Instauro la heap-property tra gli elementi " + (i+1) + " e " + (j+1) + "");
				elementoEvidenziato1 = i;
				elementoEvidenziato2 = j;
			}
			if (posMax < j) {
				
				evidenziaLinea(7);
				if (this.intervallo != INTERVALLO_TURBO) {
					setMessaggio(getMessaggio() + "Si'. " + a[posMax+1] + " > " + a[posMax] + " ? ");
					pausa(intervallo);
				} else {
					setMessaggio("Instauro la heap-property tra gli elementi " + (i+1) + " e " + (j+1) + "");
					elementoEvidenziato1 = i;
					elementoEvidenziato2 = j;
				}
				if (a[posMax+1] > a[posMax]) {
					
					evidenziaLinea(8);
					posMax++;
					if (this.intervallo != INTERVALLO_TURBO) {
						setMessaggio(getMessaggio() + "Si', quindi ora il figlio maggiore e' " + a[posMax]);
						elementoEvidenziato2 = posMax;
						pausa(intervallo);
					} else {
						setMessaggio("Instauro la heap-property tra gli elementi " + (i+1) + " e " + (j+1) + "");
						elementoEvidenziato1 = i;
						elementoEvidenziato2 = j;
					}
				} else {
					if (this.intervallo != INTERVALLO_TURBO) {
						setMessaggio(getMessaggio() + "No, quindi il figlio maggiore resta " + a[posMax]);
						elementoEvidenziato2 = posMax;
						pausa(intervallo);
					} else {
						setMessaggio("Instauro la heap-property tra gli elementi " + (i+1) + " e " + (j+1) + "");
						elementoEvidenziato1 = i;
						elementoEvidenziato2 = j;
					}
				}
			} else {
				if (this.intervallo != INTERVALLO_TURBO) {
					setMessaggio(getMessaggio() + "No, allora il figlio maggiore e' in verita' figlio unico");
					elementoEvidenziato2 = posMax;
					pausa(intervallo);
				} else {
					setMessaggio("Instauro la heap-property tra gli elementi " + (i+1) + " e " + (j+1) + "");
					elementoEvidenziato1 = i;
					elementoEvidenziato2 = j;
				}
			}
			
			evidenziaLinea(9);
			if (this.intervallo != INTERVALLO_TURBO) {
				setMessaggio(Integer.toString(a[posMax]) + " > " + value + " (l'elemento per cui cerco un posto) ? ");
				elementoEvidenziato2 = -1;
				pausa(intervallo);
			} else {
				setMessaggio("Instauro la heap-property tra gli elementi " + (i+1) + " e " + (j+1) + "");
				elementoEvidenziato1 = i;
				elementoEvidenziato2 = j;
			}
			if (a[posMax] > value) {
				
				evidenziaLinea(10);
				if (this.intervallo != INTERVALLO_TURBO) {
					setMessaggio(getMessaggio() + "Si', allora adesso considero il sottoalbero con radice " + a[posMax]);
					elementoEvidenziato2 = -1;
					pausa(intervallo);
				} else {
					setMessaggio("Instauro la heap-property tra gli elementi " + (i+1) + " e " + (j+1) + "");
					elementoEvidenziato1 = i;
					elementoEvidenziato2 = j;
				}
				a[root] = a[posMax];
				
				evidenziaLinea(11);
				root = posMax;
				
				if (this.intervallo != INTERVALLO_TURBO) {
					inserisciHeap(a,i,j,root);
				} else {
					setMessaggio("Instauro la heap-property tra gli elementi " + (i+1) + " e " + (j+1) + "");
					elementoEvidenziato1 = i;
					elementoEvidenziato2 = j;
					inserisciHeap(a,i,j,-1);
				}
			
			evidenziaLinea(12);
			} else {
				
				evidenziaLinea(13);
				if (this.intervallo != INTERVALLO_TURBO) {
					setMessaggio(getMessaggio() + "No, allora non ci sono piu' conflitti e ho trovato un posto per " + value);
					pausa(intervallo);
				} else {
					setMessaggio("Instauro la heap-property tra gli elementi " + (i+1) + " e " + (j+1) + "");
					elementoEvidenziato1 = i;
					elementoEvidenziato2 = j;
				}
				done = true;
			
			evidenziaLinea(14);
			}
			
		
		evidenziaLinea(15);
			evidenziaLinea(4);
		}
		
		evidenziaLinea(16);
		a[root] = value;
		
		elementoEvidenziato1 = elementoEvidenziato2 = -1;
	
	evidenziaLinea(17);
	}
	
	
	void makeHeap(int[] a, int n) {
		evidenziaLinea(19);
		
		evidenziaLinea(20);
		setMessaggio("Costruisco lo heap da 1 a " + (n+1));
		elementoEvidenziato1 = 0;
		elementoEvidenziato2 = n;
		pausa(intervallo);
		for(int i=(n/2);i>=0;i--) {
			if (this.intervallo != INTERVALLO_TURBO) {
				setMessaggio("Prendo come radice " + a[i]);
				elementoEvidenziato1 = i;
				pausa(intervallo);
			}
			evidenziaLinea(21);
			if (this.intervallo != INTERVALLO_TURBO) {
				setMessaggio("Instauro la heap-property tra gli elementi " + (i+1) + " e " + (n+1) + "");
				elementoEvidenziato1 = i;
				elementoEvidenziato2 = n;
				inserisciHeap(a,i,n,i);
				pausa(intervallo);
			}
			maximize(a, i, n);
			
			inserisciHeap(a,i,n,-1);
		}
	
		elementoEvidenziato1 = elementoEvidenziato2 = -1;
		
	evidenziaLinea(22);
	}
	
	void maxSort(int[] a, int n) {
		evidenziaLinea(24);
		
		evidenziaLinea(25);
		if (this.intervallo != INTERVALLO_TURBO) {
			setMessaggio("Ordino il vettore da 1 a " + (n+1));
			elementoEvidenziato1 = 0;
			elementoEvidenziato2 = n;
			pausa(intervallo);
		}
		int i = n;
		
		evidenziaLinea(26);
		while (i > 0) {
			setMessaggio("Estraggo dallo heap il massimo all'indice " + (i+1));
			elementoEvidenziato2 = i;
			pausa(intervallo);
			
			inserisciHeap(a,0,i,i);
			
			evidenziaLinea(27);
			scambia(a,0,i);
			
			evidenziaLinea(28);
			i = i-1;
			
			evidenziaLinea(29);
			if (this.intervallo != INTERVALLO_TURBO) {
				setMessaggio("Ripristino la heap-property tra il primo e il " + (i+1) + "-esimo elemento");
				elementoEvidenziato1 = 0;
				elementoEvidenziato2 = i;
				pausa(intervallo);
			}
			maximize(a, 0, i);
			
		
		evidenziaLinea(30);
		}
		
		elementoEvidenziato1 = elementoEvidenziato2 = -1;
		
	evidenziaLinea(31);
	}
	
	void heapSort(int[] a, int high) {
		evidenziaLinea(33);
		
		evidenziaLinea(34);
		makeHeap(a, high);
		
		inserisciHeap(a,0,high,-1);
		
		evidenziaLinea(35);
		maxSort(a, high);
		
		Albero.cancella();
		
		setMessaggio("Finito!");
		elementoEvidenziato1 = elementoEvidenziato2 = -1;
				
	evidenziaLinea(36);
	}
	
	public void run() {
		Albero.setScriviEtichette(true); //Cosi' l'albero mi scrive i numeri vicino ai nodi
		try {
			heapSort(vettore,vettore.length-1);
		} catch (Exception e) {} //Cosi' se il thread viene stoppato, si ferma subito
		thread = null; // <== Cosa importante da ricordare!! Cosi' quando termina il sorting, il timer viene stoppato
		Albero.setScriviEtichette(false);
	}
}