import java.awt.*;
import java.awt.geom.*;
import javax.swing.*;

public class MergeSort extends Sorting {
	
	public void stop() { //Devo anche togliere di mezzo il vettore...
		working = null;
		super.stop();
	}	
	
	public MergeSort(int[] vettore, long intervallo) {
		super(intervallo);
		xInizio = 10; yInizio = 4;
		int nElementi = vettore.length;
		this.vettore = new int[nElementi];
		for (int i=0;i<nElementi;i++) {
			this.vettore[i] = vettore[i];
		}
		String[] codice = {
			"void mergeSort(int a[], int low, int high)  {",
			"  if(low == high) ",
			"       return;",
			"  int length = high-low+1;",
			"  int pivot = (low+high) / 2;",
			"  mergesort(a, low, pivot);",
			"  mergesort(a, pivot+1, high);",
			"  int working[] = new int[length];",
			"  for(int i = 0; i < length; i++) ",
			"      working[i] = a[low+i];",
			"  int m1 = 0; ",
			"  int m2 = pivot-low+1;",
			"  for(int i = 0; i < length; i++) {",
			"    if(m2 <= high-low) ",
			"        if(m1 <= pivot-low) ",
			"            if(working[m1] > working[m2]) ",
			"                a[i+low] = working[m2++];  ",
			"            else ",
			"                a[i+low] = working[m1++];",
			"        else ",
			"            a[i+low] = working[m2++];",
			"    else ",
			"        a[i+low] = working[m1++];",
			"  }",
			"}"
		};
    	this.codice = codice;
		String[] spiegazione = {
			"Entra in MergeSort con l'array di input, e gli indici inferiore e superiore",
			"Se indice inferiore = indice superiore",
			"Ritorna al chiamante",
			"Calcola la lunghezza dell'intervallo interessato",
			"Calcola la posizione del pivot (perno)",
			"Esegui l'algoritmo sulla meta' prima del pivot",
			"Esegui l'algoritmo sulla meta' dopo il pivot",
			"Alloca il vettore di lavoro",
			"Copia la parte di vettore interessata nel vettore di lavoro",
			"Copia l'i-esimo elemento del sottovettore {low,high} nel posto i-esimo del vettore di lavoro",
			"m1 e' l'indice che scorre la prima meta' del vettore di lavoro",
			"m2 e' l'indice che scorre la seconda meta' del vettore di lavoro",
			"Copiamo ordinati gli elementi del vettore di lavoro nel sottovettore {low,high}",
			"Controlla se ci sono ancora elementi disponibili nella seconda meta' del vettore di lavoro",
			"Controlla se ci sono ancora elementi disponibili nella prima meta' del vettore di lavoro",
			"Cerca l'elemento minore tra i due minimi del vettore di lavoro",
			"L'elemento minore e' quello di indice m2",
			"Altrimenti l'elemento minore e' quello di indice m1",
			"Assegna l'elemento di indice m1 all'i-esimo posto nel sottovettore",
			"Se non ci sono piu' elementi disponibili nella prima meta' del vettore di lavoro",
			"Prendi il prossimo elemento dalla seconda meta' del vettore di lavoro",
			"Se non ci sono piu' elementi disponibili nella seconda meta' del vettore di lavoro",
			"Prendi il prossimo elemento della prima meta' del vettore di lavoro",
			"Continua il ciclo for finche' non e' stato riempito il sottovettore",
			"Fine del MergeSort"
  		};
  		this.spiegazione = spiegazione;
  	}
	
	public void draw(Graphics g1, Rectangle area) {
		this.area = area; //Per il BubbleSort l'area di disegno e' tutta quanta. Per il MergeSort no
		Graphics2D g = (Graphics2D)g1;
		this.area.height = this.area.height / 2;
		colorePrimoEvidenziato = COLORE_PRIMO_DEFAULT;
		coloreSecondoEvidenziato = COLORE_SECONDO_DEFAULT;
		disegnaVettore(g1,vettore,area,xInizio,yInizio,elementoEvidenziato1,elementoEvidenziato2,false);
		
		this.areaSotto.setRect(this.area);		
		this.areaSotto.y = this.area.y + this.area.height;
		
		g.draw(new Line2D.Double(areaSotto.x,areaSotto.y+2,areaSotto.x + areaSotto.width,areaSotto.y+2));
		
		colorePrimoEvidenziato = Color.CYAN;
		coloreSecondoEvidenziato = Color.GREEN;
		if (working != null && thread != null) {
			float xInizioVecchio = xInizio; float yInizioVecchio = yInizio;
			xInizio = 10; yInizio = 4;
			yInizio = yInizio + areaSotto.height;
			if (elementoEvidenziato1 != -1) { //Il vettore di lavoro lo disegna sotto all'altro vettore
				double distanzaX = 10;	
				double larghezza = (area.getWidth() - distanzaX)/vettore.length - distanzaX;
				xInizio = xInizio + (float)(elementoEvidenziato1 * (distanzaX+larghezza));
				disegnaVettore(g1,working,areaSotto,xInizio,yInizio,elementoEvidenziatoMerge1,elementoEvidenziatoMerge2,animazione);
			} else {
				disegnaVettore(g1,working,areaSotto,xInizio,yInizio,elementoEvidenziatoMerge1,elementoEvidenziatoMerge2,animazione);
			}
			xInizio = xInizioVecchio; yInizio = yInizioVecchio;
		}
	}
	
	private void copiaElemento(int[] vettore, int[] working, int indice, int indiceWorking,
							   int primoEvidenziato, int secondoEvidenziato) {
		
		int i = indice;
		int j = indiceWorking;

		
		if (this.intervallo != INTERVALLO_TURBO) {
			
			setMessaggio("Copio " + working[indiceWorking] + " al posto " + (indice+1));
						
			elementoEvidenziato2 = indice;
			elementoEvidenziatoMerge1 = primoEvidenziato; elementoEvidenziatoMerge2 = secondoEvidenziato;
		
			animazione = true; //Faccio presente che sto animando e quindi i vettori evidenziati vanno disegnati a parte
				
			double distanzaX = 10;
	
			double larghezza = (area.getWidth() - distanzaX)/vettore.length - distanzaX;
			double altezzaMassima = 10 * area.getHeight()/12;
			double altezzaProp = altezzaMassima / massimo(vettore);		
			
			xi = xInizio + i*(distanzaX + larghezza);  //j e' sempre quello che muovo, i e' quello che sta fermo
			yi = yInizio + altezzaMassima;
			xj = xInizio + j*(distanzaX + larghezza);
			yj = yInizio + altezzaMassima + areaSotto.y;
			xiIniziale = xi;
			xjIniziale = xj;
			yiIniziale = yi;
			yjIniziale = yj;
			
			if (elementoEvidenziato1 != -1) {
				xj += (float)(elementoEvidenziato1 * (distanzaX+larghezza));
			}

			if (elementoEvidenziatoMerge1 != -1) {
				double tmp = xi; xi = xj; xj = tmp;
				tmp = yi; yi = yj; yj = tmp;
				while (yi - yj > 0) {
					yi -= larghezza/10;
					dormi(intervallo/8);
				}
			
				int direzione = (xi > xj)?+1:-1;
			
				while (Math.abs(xi - xj) > larghezza/9) {
					xi -= direzione * larghezza/10;
					dormi(intervallo/8);
				}
			}
			
			if (elementoEvidenziatoMerge2 != -1) {
				while (yj - yi > 0) {
					yj -= larghezza/10;
					dormi(intervallo/8);
				}
			
				int direzione = (xi > xj)?+1:-1;
			
				while (Math.abs(xi - xj) > larghezza/9) {
					xj += direzione * larghezza/10;
					dormi(intervallo/8);
				}
			}
			
			animazione = false;
		} else {
			if (primoEvidenziato != -1) elementoEvidenziatoMerge1 = primoEvidenziato;
			if (secondoEvidenziato != -1) elementoEvidenziatoMerge2 = secondoEvidenziato;
		}
		
		vettore[i] = working[j];
		ListaStati.fineStato();
	}

	
	void mergeSort(int[] a, int low, int high) {
		
		evidenziaLinea(0);
		setMessaggio("Eseguo Merge Sort dall'indice " + (low+1) + " all'indice " + (high+1));
		elementoEvidenziato1 = low; elementoEvidenziato2 = high;
		pausa(intervallo);

		Nodo nodo = Albero.aggiungiNodo("MergeSort da " + (low+1) + " a " + (high+1),livello,Albero.radice);
		Albero.evidenziaNodo(nodo);
		
		evidenziaLinea(1);
		if(low == high) {
			livello--;
			evidenziaLinea(2);
			return;
		}
		
		evidenziaLinea(3);
		int length = high-low+1;
		
		evidenziaLinea(4);
		int pivot = (low+high) / 2;
		
		evidenziaLinea(5);
		livello++;
		mergeSort(a, low, pivot);
		Albero.evidenziaNodo(nodo);
		
		evidenziaLinea(6);
		livello++;
		mergeSort(a, pivot+1, high);
		Albero.evidenziaNodo(nodo);
			
		if (this.intervallo != this.INTERVALLO_TURBO) {	
			setMessaggio("Copio gli elementi da " + (low+1) + " a " + (high+1) + " nel vettore di lavoro");
			elementoEvidenziato1 = low; elementoEvidenziato2 = high;
			
			evidenziaLinea(7);
			working = new int[length];

			elementoEvidenziatoMerge1 = 0; elementoEvidenziatoMerge2 = length-1;

			evidenziaLinea(8);
			pausa(intervallo);
		} else {
			evidenziaLinea(7);
			elementoEvidenziato1 = low; elementoEvidenziato2 = high;
			elementoEvidenziatoMerge1 = 0; elementoEvidenziatoMerge2 = length-1;
			working = new int[length];
		}
			
		evidenziaLinea(8);		
		for(int i = 0; i < length; i++) {
			
			evidenziaLinea(9);
			working[i] = a[low+i];
			pausa(intervallo);
		evidenziaLinea(8);
		}
		
		evidenziaLinea(10);
		int m1 = 0;
		
		evidenziaLinea(11);
		int m2 = pivot-low+1;
		
		setMessaggio("Eseguo il merge da " + (low+1) + " a " + (high+1));
		elementoEvidenziato1 = low; elementoEvidenziato2 = high;
		elementoEvidenziatoMerge1 = m1; elementoEvidenziatoMerge2 = m2;
		evidenziaLinea(12);
		pausa(intervallo);
		for(int i = 0; i < length; i++) {
			
			int indiceCopiato = 0;
			evidenziaLinea(13);
	   		if (this.intervallo != this.INTERVALLO_TURBO) {	
		   		elementoEvidenziatoMerge2 = m2;
		   		elementoEvidenziato2 = high;
	   			setMessaggio("Ci sono ancora elementi nella seconda parte del vettore di lavoro (" + (m2+1) + " <= " + (high-low+1) + ") ? ");
	   			pausa(intervallo);
	   		}
			if(m2 <= high-low) {
				if (this.intervallo != this.INTERVALLO_TURBO) {
					setMessaggio("Si', allora...");
					pausa(intervallo);
				}
				
				evidenziaLinea(14);
				if (this.intervallo != this.INTERVALLO_TURBO) {
					elementoEvidenziatoMerge1 = m1;
					elementoEvidenziato2 = pivot-low;
					setMessaggio("Ci sono ancora elementi nella prima parte del vettore di lavoro (" + m1 + " <= " + (pivot-low+1) + ") ? ");
					pausa(intervallo);
				}
				if(m1 <= pivot-low) {
					if (this.intervallo != this.INTERVALLO_TURBO) {
						setMessaggio("Si', allora ci metto l'elemento minore");
						pausa(intervallo);
					}
					
					evidenziaLinea(15);
					if (this.intervallo != this.INTERVALLO_TURBO) {
						elementoEvidenziatoMerge1 = m1;
						elementoEvidenziatoMerge2 = m2;
						setMessaggio(Integer.toString(working[m1]) + " < " + working[m2] + " ? ");
						pausa(intervallo);
					}
					if(working[m1] > working[m2]) {
						
						evidenziaLinea(16);
						if (this.intervallo != this.INTERVALLO_TURBO) {
							setMessaggio("Si', allora e' l'elemento minore e lo copio");
							pausa(intervallo);
						}
						indiceCopiato = m2;
						//a[i+low] = working[m2++];
						copiaElemento(a,working,i+low,m2++,-1,indiceCopiato);
						
					evidenziaLinea(17);
					} else {
						if (this.intervallo != this.INTERVALLO_TURBO) {
							setMessaggio("No, allora l'elemento minore e' " + working[m1] + " e copio quello");
							pausa(intervallo);
						}
						
						evidenziaLinea(18);
						indiceCopiato = m1;
						//a[i+low] = working[m1++];
						copiaElemento(a,working,i+low,m1++,indiceCopiato,-1);
					}
					
				evidenziaLinea(19);
				} else {
					if (this.intervallo != this.INTERVALLO_TURBO) {
						setMessaggio("No, posso solo mettere " + working[m2] + ", che e' sicuramente l'elemento minore");
						pausa(intervallo);
					}
					
					evidenziaLinea(20);
					indiceCopiato = m2;
					//a[i+low] = working[m2++];
					copiaElemento(a,working,i+low,m2++,-1,indiceCopiato);
				}
				
			evidenziaLinea(21);
			} else {
				if (this.intervallo != this.INTERVALLO_TURBO) {
					setMessaggio("No, posso solo mettere " + working[m1] + ", che e' sicuramente l'elemento minore");
					pausa(intervallo);
				}
				
				evidenziaLinea(22);
				indiceCopiato = m1;
				//a[i+low] = working[m1++];
				copiaElemento(a,working,i+low,m1++,indiceCopiato,-1);
			}
			
			/*if (this.intervallo != this.INTERVALLO_TURBO) {	
				setMessaggio("Copio " + working[indiceCopiato] + " al posto " + (i+low+1));
				elementoEvidenziato1 = low; elementoEvidenziato2 = i+low;
				if (indiceCopiato == m1-1) elementoEvidenziatoMerge1 = indiceCopiato;
				if (indiceCopiato == m2-1) elementoEvidenziatoMerge2 = indiceCopiato;
			}*/
			pausa(intervallo);
			
		evidenziaLinea(23);
		}
	evidenziaLinea(24);
		setMessaggio("Ho finito Merge Sort da " + (low+1) + " a " + (high+1));
		elementoEvidenziato1 = low; elementoEvidenziato2 = high;
		working = null;
		pausa(intervallo);
		livello--;
	}
			
	public void run() {	
		livello = 0;
		Albero.cancella();
		try {
			mergeSort(vettore,0,vettore.length-1);
		} catch(Exception e) {} //Cosi' se si fa stop, il thread viene stoppato subito
		thread = null; // <== Cosa importante da ricordare!! Cosi' quando termina il sorting, il timer viene stoppato
	}
	
	private int[] working = null;
	private Rectangle areaSotto = new Rectangle();
	private int elementoEvidenziatoMerge1 = -1;
	private int elementoEvidenziatoMerge2 = -1;
	private int livello = 0;
}