import java.awt.*;
import javax.swing.*;

public class QuickSort extends Sorting {
	
	public QuickSort(int[] vettore, long intervallo) {
		super(intervallo);
		int nElementi = vettore.length;
		this.vettore = new int[nElementi];
		for (int i=0;i<nElementi;i++) {
			this.vettore[i] = vettore[i];
		}
		
		String[] codice = {			
			"void quickSort(int[] a, int low, int high) {",
			"    if (low >= high)",
			"        return;",
			"    int pivot = a[low];",
			"    int i = low;",
			"    int j = high;",
			"    while (i < j) {",
			"        while (a[i] <= pivot && i < high)",
			"            i += 1;",
			"        while (a[j] >= pivot && j > low)",
			"            j -= 1;",
			"        if (i < j)",
			"            swap(i,j);",
			"    }",
			"    if (low < j)",
			"        swap(low,j);",
			"    if (j < low)",
			"        swap(j,low);",
			"    quicksort(low,j-1);",
			"    quicksort(j+1,high);",
			"}"
		};
		this.codice = codice;
		
		String[] spiegazione = {
			"Entra in QuickSort con l'array di input, e gli indici inferiore e superiore",
			"Se indice inferiore >= indice superiore",
			"Ritorna al chiamante",
			"Sceglie per pivot (perno) il primo elemento della parte di vettore considerata",
			"i scorre fino all'indice dell'ultimo elemento <= al pivot (la parte di vettore trovata e' tutta <= al pivot e viene prima del pivot)",
			"j scorre fino all'indice dell'ultimo elemento >= al pivot (la parte di vettore trovata e' tutta >= al pivot e viene dopo il pivot)",
			"Continua finche' i e j non si sono invertiti",
			"Esegui il ciclo finche' i puntera' ad elementi <= al pivot e non uscira' dal vettore",
			"Fa puntare i al prossimo elemento",
			"Esegui il ciclo finche' j puntera' ad elementi >= al pivot e non uscira' dal vettore",
			"Fa puntare j al prossimo elemento",
			"Se i e j si sono invertiti",
			"Scambia gli elementi puntati da i e j",
			"Continua finche' i e' < j",
			"Se j e' sopra l'indice inferiore",
			"Scambia gli elementi low e j",
			"Se j e' sotto l'indice inferiore",
			"Scambia gli elementi low e j",
			"Richiama quicksort da low fino a j-1",
			"Richiama quicksort da j+1 a high",
			"Fine del QuickSort"
		};
		this.spiegazione = spiegazione;
	}
	
   	public void draw(Graphics g1, Rectangle area) {
		this.area = area; //Per il QuickSort l'area di disegno e' tutta quanta
		Graphics2D g = (Graphics2D)g1;
		disegnaVettore(g1,vettore,area,xInizio,yInizio,elementoEvidenziato1,elementoEvidenziato2,animazione);
	}
	
	void quickSort(int[] a, int low, int high) {
		//System.out.println("Quick sort da " + (low+1) + " a " + (high+1) + ": livello " + livello);
		setMessaggio("Inizio Quick Sort dall'indice " + (low+1) + " all'indice " + (high+1));
		evidenziaLinea(0);
		elementoEvidenziato1 = low; elementoEvidenziato2 = high;
		pausa(intervallo);
				
		Nodo nodo = Albero.aggiungiNodo("QuickSort da " + (low+1) + " a " + (high+1),livello,Albero.radice);
		Albero.evidenziaNodo(nodo);
	
		evidenziaLinea(1);
		if (low >= high) {
		
			livello--;
			evidenziaLinea(2);
			return;
		}
		
		evidenziaLinea(3);
		setMessaggio("Il pivot e' " + a[low]);
		elementoEvidenziato1 = low;
		pausa(intervallo);
		int pivot = a[low];
		
		evidenziaLinea(4);
		int i = low;
		
		evidenziaLinea(5);
		int j = high;
		
		evidenziaLinea(6);
		while (i < j) {
		
			evidenziaLinea(7);
			if (this.intervallo != this.INTERVALLO_TURBO) {
				setMessaggio("Cerco gli elementi minori del pivot ( " + pivot + " ) partendo da " + (i+1));
				elementoEvidenziato1 = i;
				pausa(intervallo);
			} else {
				setMessaggio("Il pivot e' " + a[low]);
				elementoEvidenziato1 = low;
			}

	        if (this.intervallo != this.INTERVALLO_TURBO) {
	        	setMessaggio(Integer.toString(a[i]) + " <= " + pivot + " (pivot) e non sono ancora in fondo al vettore ? ");
	        	elementoEvidenziato1 = i;
	        	pausa(intervallo);
	        } else {
				setMessaggio("Il pivot e' " + a[low]);
				elementoEvidenziato1 = low;
			}
		    while (a[i] <= pivot && i < high) {
		    	
		    	evidenziaLinea(8);
		        i += 1;
		        
		        if (this.intervallo != this.INTERVALLO_TURBO) {
		        	setMessaggio(Integer.toString(a[i]) + " <= " + pivot + " (pivot) e non sono ancora in fondo al vettore ? ");
		        	elementoEvidenziato1 = i;
		        	evidenziaLinea(7);
		        	pausa(intervallo);
		        } else {
					setMessaggio("Il pivot e' " + a[low]);
					elementoEvidenziato1 = low;
				}
		    }
		    
		    if (this.intervallo != this.INTERVALLO_TURBO) {
		    	if (i >= high) setMessaggio(getMessaggio() + "No, sono in fondo al vettore");
		    	else if (a[i] > pivot) setMessaggio(getMessaggio() + "No, " + a[i] + " > " + pivot);
		    	elementoEvidenziato1 = i;
		    	pausa(intervallo);
		    }
		    
		    evidenziaLinea(9);
			if (this.intervallo != this.INTERVALLO_TURBO) {
				setMessaggio("Cerco gli elementi maggiori del pivot ( " + pivot + " ) partendo da " + (j+1));
				elementoEvidenziato2 = j;
				pausa(intervallo);
			} else {
				setMessaggio("Il pivot e' " + a[low]);
				elementoEvidenziato1 = low;
			}

	        if (this.intervallo != this.INTERVALLO_TURBO) {
	        	setMessaggio(Integer.toString(a[j]) + " >= " + pivot + " (pivot) e non sono ancora in cima al vettore ? ");
	        	elementoEvidenziato2 = j;
	        	pausa(intervallo);
	        } else {
				setMessaggio("Il pivot e' " + a[low]);
				elementoEvidenziato1 = low;
			}
		    while (a[j] >= pivot && j > low) {
		    	
		    	evidenziaLinea(10);
		        j -= 1;
		        
		        if (this.intervallo != this.INTERVALLO_TURBO) {
		        	setMessaggio(Integer.toString(a[j]) + " >= " + pivot + " (pivot) e non sono ancora in cima al vettore ? ");
		        	elementoEvidenziato2 = j;
		        	evidenziaLinea(9);
		        	pausa(intervallo);
		        } else {
					setMessaggio("Il pivot e' " + a[low]);
					elementoEvidenziato1 = low;
				}
		    }
		    
		    if (this.intervallo != this.INTERVALLO_TURBO) {
		    	if (j <= low) setMessaggio(getMessaggio() + "No, sono in cima al vettore");
		    	else if (a[j] < pivot) setMessaggio(getMessaggio() + "No, " + a[j] + " < " + pivot);
		    	elementoEvidenziato2 = j;
		    	pausa(intervallo);
		    } else {
				setMessaggio("Il pivot e' " + a[low]);
				elementoEvidenziato1 = low;
			}
		    
		    evidenziaLinea(11);
		    if (this.intervallo != this.INTERVALLO_TURBO) {
		    	setMessaggio(Integer.toString(i+1) + " < " + (j+1) + " ? ");
		    	pausa(intervallo);
		    } else {
				setMessaggio("Il pivot e' " + a[low]);
				elementoEvidenziato1 = low;
			}
		    if (i < j) {
		    	
		    	evidenziaLinea(12);
		    	if (this.intervallo != this.INTERVALLO_TURBO) {
		    		setMessaggio(getMessaggio() + "Si', allora li scambio");
		    		pausa(intervallo);
		    	} else {
					setMessaggio("Il pivot e' " + a[low]);
					elementoEvidenziato1 = low;
				}
		        scambia(a,i,j);
		    } else {
		    	if (this.intervallo != this.INTERVALLO_TURBO) {
		    		setMessaggio(getMessaggio() + "No, procedo");
		    		pausa(intervallo);
		    	} else {
					setMessaggio("Il pivot e' " + a[low]);
					elementoEvidenziato1 = low;
				}
		    }
		    
		evidenziaLinea(13);
		}
		
		evidenziaLinea(14);
		if (this.intervallo != this.INTERVALLO_TURBO) {
			setMessaggio("L'indice che scorre gli elementi maggiori (j) e' sopra l'estremo inferiore ( " + (j+1) + " > " + (low+1) + " ) ? ");
			pausa(intervallo);
		} else {
				setMessaggio("Il pivot e' " + a[low]);
				elementoEvidenziato1 = low;
			}
		if (low < j) {
			
			evidenziaLinea(15);
			if (this.intervallo != this.INTERVALLO_TURBO) {
				setMessaggio(getMessaggio() + "Si', allora li scambio");
				pausa(intervallo);
			} else {
				setMessaggio("Il pivot e' " + a[low]);
				elementoEvidenziato1 = low;
			}
			scambia(a,low,j);
		} else {
			if (this.intervallo != this.INTERVALLO_TURBO) {
				setMessaggio(getMessaggio() + "No, procedo");
				pausa(intervallo);
			} else {
				setMessaggio("Il pivot e' " + a[low]);
				elementoEvidenziato1 = low;
			}
		}
		
		evidenziaLinea(16);
		if (this.intervallo != this.INTERVALLO_TURBO) {
			setMessaggio("L'indice che scorre gli elementi maggiori (j) e' sotto l'estremo inferiore ( " + (j+1) + " < " + (low+1) + " ) ? ");
			pausa(intervallo);
		} else {
			setMessaggio("Il pivot e' " + a[low]);
			elementoEvidenziato1 = low;
		}
		if (j < low) {
			
			evidenziaLinea(17);
			if (this.intervallo != this.INTERVALLO_TURBO) {
				setMessaggio(getMessaggio() + "Si', allora li scambio");
				pausa(intervallo);
			} else {
				setMessaggio("Il pivot e' " + a[low]);
				elementoEvidenziato1 = low;
			}
			scambia(a,j,low);
		} else {
			if (this.intervallo != this.INTERVALLO_TURBO) {
				setMessaggio(getMessaggio() + "No, procedo");
				pausa(intervallo);
			} else {
				setMessaggio("Il pivot e' " + a[low]);
				elementoEvidenziato1 = low;
			}
		}
		
		livello++;
		evidenziaLinea(18);
		quickSort(a,low,j-1);
		Albero.evidenziaNodo(nodo);
		
		livello++;
		evidenziaLinea(19);
		quickSort(a,j+1,high);
		Albero.evidenziaNodo(nodo);
	
	evidenziaLinea(20);
		setMessaggio("Ho finito il QuickSort da " + (low+1) + " a " + (high+1));
		elementoEvidenziato1 = low; elementoEvidenziato2 = high;
		pausa(intervallo);
		livello--;
	}
	
	public void run() {
		livello = 0;
		Albero.cancella();
		try {
			quickSort(vettore,0,vettore.length-1);
		} catch (Exception e) {} //Cosi', se il thread viene stoppato, si ferma subito
		thread = null; // <== Cosa importante da ricordare!! Cosi' quando termina il sorting, il timer viene stoppato
	}
	
	private int livello = 0;

}